<?php

namespace Drupal\file_url\Element;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\file\Element\ManagedFile;
use Drupal\file_url\Entity\RemoteFile;
use Drupal\file_url\FileUrlHandler;

/**
 * Provides an AJAX/progress aware widget for uploading and saving a file URL.
 *
 * @FormElement("file_url")
 */
class FileUrl extends ManagedFile {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    return [
      // Either 'upload', or 'remote'.
      '#file_mode' => NULL,
    ] + parent::getInfo();
  }

  /**
   * {@inheritdoc}
   */
  public static function processManagedFile(&$element, FormStateInterface $form_state, &$complete_form) {
    // Dereference values.
    $default_value = $element['#default_value'];
    $value =& $element['#value'];
    $type = NULL;
    if (!empty($default_value['target_id'])) {
      $file = FileUrlHandler::urlToFile($default_value['target_id']);
      if (FileUrlHandler::isRemote($file)) {
        $value['fids'] = [$default_value['target_id']];
        $type = 'remote';
        $element['remote'] = [
          '#type' => 'hidden',
          '#value' => $default_value['target_id'],
        ];
      }
      else {
        $value['target_id'] = $file->id();
        $value['fids'] = [$file->id()];
        $type = 'upload';
      }
      $element['type'] = [
        '#type' => 'hidden',
        '#value' => $type,
      ];
    }

    // Call parent 'managed_file' process method.
    $element = parent::processManagedFile($element, $form_state, $complete_form);

    $delta = $element['#delta'];

    // Show the form elements only on new item sub-form.
    $fids = isset($value['fids']) ? $value['fids'] : [];
    if (!$fids) {
      $element['type'] = [
        '#type' => 'radios',
        '#options' => [
          'upload' => t('Upload file'),
          'remote' => t('Remote file URL'),
        ],
        '#weight' => -12,
        '#prefix' => '<div class="container-inline">',
        '#suffix' => '</div>',
      ];

      $field_name = $element['#field_name'];
      $selector = ':input[name="' . $field_name . '[' . $delta . '][type]"]';
      $remote_visible = [$selector => ['value' => 'remote']];
      $element['remote'] = [
        '#type' => 'url',
        '#title' => t('Remote URL'),
        '#description' => t('This must be an external URL such as <em>http://example.com</em>.'),
        // Only show this field when the 'remote' radio is selected.
        '#states' => ['visible' => $remote_visible],
        '#attached' => [
          'library' => ['file_url/remote_url'],
        ],
        '#weight' => -7,
        '#attributes' => [
          'data-drupal-file-url-remote' => TRUE,
        ],
      ];

      // Only show this field when the 'upload' radio is selected. Add also a
      // wrapper around file upload, so states knows what field to target.
      $upload_visible = [$selector => ['value' => 'upload']];
      $element['upload']['#states']['visible'] = $upload_visible;
      $element['upload']['#theme_wrappers'][] = 'form_element';

      // Change the 'upload_button' label, if case.
      if ($type === 'remote') {
        $element['upload_button']['#value'] = t('Add remote URL');
      }
    }

    // Render also the remote files.
    if ($type === 'remote') {
      $remote_file = RemoteFile::load($default_value['target_id']);
      $element['#files'] = [$default_value['target_id'] => $remote_file];
      $file_link = [
        '#type' => 'link',
        '#title' => $remote_file->getFileUri(),
        '#url' => Url::fromUri($remote_file->getFileUri()),
      ];
      if ($element['#multiple']) {
        $element['file_' . $delta]['selected'] = [
          '#type' => 'checkbox',
          '#title' => \Drupal::service('renderer')->renderPlain($file_link),
        ];
      }
      else {
        $element['file_' . $delta]['filename'] = $file_link + ['#weight' => -10];
      }
    }

    // The upload instructions are added directly to the file upload element.
    $element['upload']['#description'] = [
      '#theme' => 'file_upload_help',
      '#description' => '',
      '#upload_validators' => $element['#upload_validators'],
      '#cardinality' => $element['#cardinality'],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function validateManagedFile(&$element, FormStateInterface $form_state, &$complete_form) {
    $fid = isset($element['fids']['#value'][0]) ? $element['fids']['#value'][0] : NULL;
    if ($fid && ctype_digit($fid)) {
      // If it's a file upload, pass it to the 'managed_file' validation.
      parent::validateManagedFile($element, $form_state, $complete_form);
    }
  }

}
