<?php

namespace Drupal\file_url\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\file\Plugin\Field\FieldWidget\FileWidget;
use Drupal\file_url\FileUrlHandler;

/**
 * Plugin implementation of the 'file_url_generic' widget.
 *
 * @FieldWidget(
 *   id = "file_url_generic",
 *   label = @Translation("File URL"),
 *   field_types = {
 *     "file_url"
 *   }
 * )
 */
class FileUrlWidget extends FileWidget {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'add_new_label' => 'Upload a new file or enter a URL',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);

    // Allow configuring only if the field has multiple values.
    if ($this->fieldDefinition->getFieldStorageDefinition()->getCardinality() !== 1) {
      $element['add_new_label'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Label for new item form'),
        '#default_value' => $this->getSetting('add_new_label'),
      ];
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    // Using a custom 'managed_file', called 'file_url'.
    $element_info = $this->elementInfo->getInfo('file_url');
    $element['#type'] = 'file_url';
    $element['#process'] = array_merge($element_info['#process'], [[static::class, 'process']]);
    $element['#value_callback'] = [static::class, 'value'];

    if ($this->fieldDefinition->getFieldStorageDefinition()->getCardinality() === 1) {
      // On single fields show only the field description. Upload instructions
      // were moved to 'file_url' element.
      $element['#description'] = $this->getFilteredDescription();
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function formMultipleElements(FieldItemListInterface $items, array &$form, FormStateInterface $form_state) {
    $elements = parent::formMultipleElements($items, $form, $form_state);

    // Use the configurable 'new item' label.
    $elements['#file_upload_title'] = $this->getSetting('add_new_label');
    // The upload instructions are added directly to 'file_url' element.
    unset($elements['#file_upload_description']);

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $values = parent::massageFormValues($values, $form, $form_state);

    // Convert file IDs to URLs.
    foreach ($values as &$value) {
      if (ctype_digit($value['target_id'])) {
        $file = File::load((int) $value['target_id']);
        $value['target_id'] = FileUrlHandler::fileToUrl($file);
      }
    }

    return $values;
  }

  /**
   * {@inheritdoc}
   */
  public static function value($element, $input, FormStateInterface $form_state) {
    $return = parent::value($element, $input, $form_state);

    $remote_url = NULL;
    if (!empty($return['remote'])) {
      $remote_url = $return['remote'];
    }
    elseif (!empty($element['#default_value']['remote'])) {
      $remote_url = $element['#default_value']['remote'];
    }

    if ($remote_url) {
      $return['fids'] = [$remote_url];
      $return['target_id'] = $remote_url;
    }

    return $return;
  }

}
